*Normalized by sum of budget shares;
*Macro used to create distance measures based on product attributes;
%macro aspace;

proc sql;
  select max(MID) into:NM trimmed from allbev;
  quit;
%put a1=|&NM|;

*indicator variable for whether 2nd set of continuous measures are used;
%if &nca2>0 %then %let dnca2 = 1; %else %let dnca2 = 0;

data spacedata;
set _null_;
run;

%do m = 1 %to &NM;

  data temp ;
  set allbev;
  if MID ne &m then delete;
  TPID+1;            *Assign a temporary product ID;
  run;

  proc sql;
    select max(TPID) into:TNP trimmed from temp;
    quit;
  %put a2=|&TNP|;    *Number of products in a market/quarter ;

  data onemkt;
  set _null_;
  run;
 
  %do n = 1 %to &TNP;
 
    data own;
    set temp;
    if TPID ne &n then delete; 
    %if (&nda+&nda2) gt 0 %then %do k = 1 %to (&nda+&nda2);
     R%scan((%dalist %dalist2),&k) = %scan((%dalist %dalist2),&k); keep R%scan((%dalist %dalist2),&k);
    %end;
    %if (&nca+&nca2) gt 0 %then %do k = 1 %to (&nca+&nca2);
     R%scan((%calist %calist2),&k) = %scan((%calist %calist2),&k); keep R%scan((%calist %calist2),&k);
    %end;
    Rwh = wh;
    Rw0 = w0;
    Rw = w;
    keep MID Rwh Rw0 Rw;
    run;

    data other;
    set temp;
    if TPID = &n then delete;
    run;

    *Create product space measures;
    data cross;
    merge own other;
    by MID;      
    *Discrete measures;
    %if &nda ne 0 %then %do k = 1 %to &nda;
	if R%scan(%dalist,&k) = %scan(%dalist,&k) and R%scan(%dalist,&k) ne " " and %scan(%dalist,&k) ne " " 
       then do; D%scan(%dalist,&k) = w; H%scan(%dalist,&k) = wh; B%scan(%dalist,&k) = w0; A%scan(%dalist,&k) = 1; end; 
       else do; D%scan(%dalist,&k) = 0; H%scan(%dalist,&k) = 0; B%scan(%dalist,&k) = 0; A%scan(%dalist,&k) = 0; end;
    %end;
    %if &nda2 ne 0 %then %do k = 1 %to &nda2;
	if R%scan(%dalist2,&k) ne %scan(%dalist2,&k)  
       then do; D%scan(%dalist2,&k) = w; H%scan(%dalist2,&k) = wh; B%scan(%dalist2,&k) = w0; A%scan(%dalist2,&k) = 1; end; 
       else do; D%scan(%dalist2,&k) = 0; H%scan(%dalist2,&k) = 0; B%scan(%dalist2,&k) = 0;  A%scan(%dalist2,&k) = 0; end;
    %end;
    *one-way Continuous measures;
    %if &nca ne 0 %then %do k = 1 %to &nca;
      if R%scan(%calist,&k) ne 888 and %scan(%calist,&k) ne 888 then 
       C%scan(%calist,&k) = 1/(1+2*abs(R%scan(%calist,&k)-%scan(%calist,&k)));   
      else C%scan(%calist,&k) = 0;
    %end;   
    *two-way Continuous measures;
    %if &nca2 ne 0 %then %do;
       denom = 1;
       %do k = 1 %to &nca2;
        denom = denom + 2*abs(R%scan(%calist2,&k)-%scan(%calist2,&k));
       %end;
       C%scan(%calist2,1) = 1/denom;
    %end;
    run;

    proc summary data = cross;
    var %if (&nda+&nda2) gt 0 %then %do k = 1 %to (&nda+&nda2); 
          D%scan((%dalist %dalist2),&k) H%scan((%dalist %dalist2),&k) B%scan((%dalist %dalist2),&k) A%scan((%dalist %dalist2),&k) %end;;
    by MID;
    output out = basew 
      sum = %if (&nda+&nda2) gt 0 %then %do k = 1 %to (&nda+&nda2); 
               BD%scan((%dalist %dalist2),&k) BH%scan((%dalist %dalist2),&k) BB%scan((%dalist %dalist2),&k) BA%scan((%dalist %dalist2),&k) %end;;
    run;

    data cross;
    merge cross basew;
    by MID;
    drop _TYPE_ _FREQ_;
    *Rival price times attribute distance;
    *Discrete distance;
    %if (&nda+&nda2) gt 0 %then %do k = 1 %to (&nda+&nda2);
        P_D%scan((%dalist %dalist2),&k) = log(p)*D%scan((%dalist %dalist2),&k)/(BD%scan((%dalist %dalist2),&k)+Rw);
        nevop_D%scan((%dalist %dalist2),&k) = log(nevop)*B%scan((%dalist %dalist2),&k)/(BB%scan((%dalist %dalist2),&k)+Rw0);
        P_H%scan((%dalist %dalist2),&k) = log(p)*H%scan((%dalist %dalist2),&k)/(BH%scan((%dalist %dalist2),&k)+Rwh);
        P_B%scan((%dalist %dalist2),&k) = log(p)*B%scan((%dalist %dalist2),&k)/(BB%scan((%dalist %dalist2),&k)+Rw0);
        P_A%scan((%dalist %dalist2),&k) = log(p)*A%scan((%dalist %dalist2),&k)/(BA%scan((%dalist %dalist2),&k)+1);
        nevop_A%scan((%dalist %dalist2),&k) = log(nevop)*A%scan((%dalist %dalist2),&k)/(BA%scan((%dalist %dalist2),&k)+1);
    %end;
    *one-way and two-way Continuous distance;
    %if (&nca+&dnca2) gt 0 %then %do k = 1 %to (&nca+&dnca2);
       P_C%scan((%calist %calist2),&k) = log(p)*C%scan((%calist %calist2),&k);
       nevop_C%scan((%calist %calist2),&k) = log(nevop)*C%scan((%calist %calist2),&k);
    %end;
    run;

    *Aggregate rival product prices into single price;
    proc summary data = cross;
    var %pr %nevopr; 
    by MID;
    output out = rival_onemkt_onepdt sum = ;
    run;

    *Own product price, quantity, dollar;
    data own_onemkt_onepdt;
    set temp;
    if TPID ne &n then delete;
    run;

    *Merge own and rival price together;
    data onemkt_onepdt;
    merge own_onemkt_onepdt rival_onemkt_onepdt;
    by MID;
    run;

    *Stack all products in one market;
    data onemkt;
    set onemkt onemkt_onepdt;
    drop TPID _TYPE_ _FREQ_;       
    run;

  %end;
  
  data spacedata;
  set spacedata onemkt;
  run;

%end;

%mend;
