*AJAE revision;
*Quasi-FMDM with Engel, Cournot, and Euler aggregation imposed;
*calculate elasticities using Green and Alston (1990, AJAE) formula;
*homogeneity and adding-up imposed;
*suitable for DM unweighted or weighted by base share;
%macro e_ga_r;

*draw parameters from the var-cov matrices for top and second-level demand models;
%rand;

*this will be where elasticities are saved;
data allmkte;
set _null_;
run;

*this will be where per capita cv, tax, energy are saved;
data allmktq;
set _null_;
run;

*indicator variable for whether 2nd set of continuous measures are used;
%if &nca2>0 %then %let dnca2 = 1; %else %let dnca2 = 0;

%do m = 1 %to &NM;
 data temp; set aid;
 if MID ne &m then delete;
 TPID+1;       *Assign a temporary product ID;
 run;

 data _null_;
 set temp end=lastobs;
 if lastobs=1 then call symput('TNP', TPID);  *number of products in a market/quarter;
 run;
 %put there are &TNP brands in market &m;

 %if (&nda+&nda2) gt 0 %then %do i = 1 %to (&nda+&nda2);
  data ws&i._onemkt; set _null_; run;
 %end;

 %if (&nca+&dnca2) gt 0 %then %do i = 1 %to (&nca+&dnca2);
  data wc&i._onemkt; set _null_; run;
 %end;

 *Prepare datasets for simulation;
 %do n = 1 %to &TNP; 
   %mat_ga; 
 %end;     

 data all; length LPID $6.; set all; LPID = compress('PID'||PID); run;  

 *column label;
 data CLB; set all; keep MID PID; run;

 proc transpose data = all out = LBL; var LPID; run;

 data LBL; set LBL; drop _NAME_; run;

 proc sql noprint;
 select pid into: x_parm separated by ' b' from all;
 select (pid+&NP+(&nda+&nda2+&nca+&dnca2)) into: p_parm separated by ' b' from all;
 quit;

 %let fb=b;

 *coefficients on group expenditure;
 data x_parm; set sim_parm2(keep=&fb&x_parm); run;

 *coefficients on own prices;
 data p_parm; set sim_parm2(keep=&fb&p_parm); run;

 *coefficients on cross prices;
 data c_parm; set sim_parm2(keep=b%eval(&NP+1)-b%eval(&NP+(&nda+&nda2+&nca+&dnca2))); run;

 *simulate pricing strategies;
 proc iml;

  use x_parm; read all into bet;
  use p_parm; read all into gamii;
  use c_parm; read all into gamij;

  use all; read all var {p} into p;
  use all; read all var {p0_oz} into p0_oz;
  use all; read all var {calorie} into calorie;
  use all; read all var {vol} into vol;
  use all; read all var {tid1} into tid1;
  use all; read all var {w} into w;
  use all; read all var {w0} into w0;

  use all; read point 1 var {uno mkt7 mkt32 mkt49 qw2 qw3 qw4 qw5 qw6 qw7 qw8 qw9 qw10 
    qw11 qw12 qw13 yr08 yr09 yr10 yr11} into shifters;
  use all; read point 1 var {pop} into pop;
  use all; read point 1 var {pcx} into pcx;
  use all; read point 1 var {y} into y;      
  use all; read point 1 var {cpi} into cpi;  
  use all; read point 1 var {p0_oz_all} into p0_oz_all;
  use all; read point 1 var {lnp_oth} into lnp_oth;

  *import estimates of top-level demand;
  use sim_parm1; read all var {a10 a1_mkt7 a1_mkt32 a1_mkt49 a1_qw2 a1_qw3 a1_qw4 a1_qw5 
    a1_qw6 a1_qw7 a1_qw8 a1_qw9 a1_qw10 a1_qw11 a1_qw12 a1_qw13 a1_yr08 a1_yr09 a1_yr10 
    a1_yr11} into shift_parms;
  use sim_parm1; read all var {g11} into g11;
  use sim_parm1; read all var {c1} into c1;

  *top-level demand theoretical restrictions;
  a0 = 0;
  a1 = shift_parms*shifters`;
  a2 = J(1+&nsm,1,1) - a1;
  g12 = J(1+&nsm,1,0) - g11;
  g22 = J(1+&nsm,1,0) - g12;
  c2 = J(1+&nsm,1,0) - c1;

  use LBL; read all var _CHAR_ into LB;
  use CLB; read all into CL;

  *baseline price index;
  p_bev = exp(w`*log(p));  
  lnp_bev = log(p_bev);
  
  *baseline per capita liquid ounce;
  q0 = vol/pop;

  *baseline per capita calories;
  engy0 = (calorie/8)`*q0;  

  *first row contains simulated output using parameter point estimate;
  *last nsm rows contains simulated outputs using random parameter draws;
  sout = J(1+&nsm,25,0);

  *first row contains row sums of gamma for all budget share equations plus col sum of beta;
  hout = J(1+&nsm,ncol(bet),0);
  aout = J(1+&nsm,1,0);

  do n = 1 to (1+&nsm);

   *overall beverage price elasticity;
   e_b = -1 + (g11[n,] - c1[n,]*(a1[n,] + g11[n,]*lnp_bev + g12[n,]*lnp_oth))/(pcx/y); 

   gamma = J(ncol(bet),ncol(bet),0);
   U = diag(1/w)*bet[n,]`;
   V = w`*diag(log(p));

   %if &mv=1 %then %do;
     %if (&nda+&nda2) gt 0 %then %do i = 1 %to (&nda+&nda2);
       use ws&i._onemkt; read all into ws&i;
       gamma = gamma + gamij[n,&i]*ws&i;
     %end;
     %if (&nca+&dnca2) gt 0 %then %do i = 1 %to (&nca+&dnca2);
       use wc&i._onemkt; read all into wc&i;                  
       gamma = gamma + gamij[n,&nda+&nda2+&i]*wc&i;
     %end;
   %end;

   %if &mv=2 %then %do;
     %if (&nda+&nda2) gt 0 %then %do i = 1 %to (&nda+&nda2);
       use ws&i._onemkt; read all into ws&i;
       gamma = gamma + diag(w0)*gamij[n,&i]*ws&i;
     %end;
     %if (&nca+&dnca2) gt 0 %then %do i = 1 %to (&nca+&dnca2);
       use wc&i._onemkt; read all into wc&i;
       gamma = gamma + gamij[n,&nda+&nda2+&i]*wc&i;
     %end;
   %end;

   gamma = diag(gamii[n,])+gamma;

   *testing homogeneity and adding up;
   homo = J(1,ncol(bet),1)*gamma;
   addup = bet[n,]*J(ncol(bet),1,1);

   A = -I(ncol(bet)) + diag(1/w)*(gamma - bet[n,]`*w`);  

   *conditional price elasticities;
   H = inv(U*V + I(ncol(bet)))*(A + I(ncol(bet))) - I(ncol(bet));  

   *expenditure elasticities;
   E = inv(U*V + I(ncol(bet)))*U + J(ncol(bet),1,1);    

   *homogeneity through Euler aggregation;
   H_lst_c = -(H[1:ncol(bet)-1,1:ncol(bet)-1]*J(ncol(bet)-1,1,1) + E[1:ncol(bet)-1,]);

   *Cournot aggregation;
   H_lst_r = -(w` + w[1:ncol(bet)-1,]`*(H[1:ncol(bet)-1,1:ncol(bet)-1]||H_lst_c))/w[ncol(bet),];

   H = (H[1:ncol(bet)-1,1:ncol(bet)-1]||H_lst_c)//H_lst_r;

   *adding up through Engel aggregation;
   E_lst = (1-w[1:ncol(bet)-1,]`*E[1:ncol(bet)-1,])/w[ncol(bet),];

   E = E[1:ncol(bet)-1,]//E_lst;

   E_xp = (1 + e_b)* (w` + V*(H + I(ncol(bet))))/(1 - (1 + e_b)*V*(E - J(ncol(bet),1,1))); 

   *unconditional elasticities;
   H_u = H + E*E_xp;

   *half cent per ounce tax, $/ounce equivalent;
   tax1 = 0.005*tid1;      

   *0.04 cent per calorie tax, $/ounce equivalent;
   tax2 = 0.0004*calorie#tid1/8;

   *percent change in beverage group price index;
   *first-order effect, direct effect;
   dlnp_bev10 = w`*(tax1/p0_oz/p);   
   dlnp_bev20 = w`*(tax2/p0_oz/p);   

   *second-order effect, substitution effects;
   dlnp_bev11 = V*(H + I(ncol(bet)) + (E - J(ncol(bet),1,1))*E_xp)*(tax1/p0_oz/p);         
   dlnp_bev21 = V*(H + I(ncol(bet)) + (E - J(ncol(bet),1,1))*E_xp)*(tax2/p0_oz/p);         

   *total effect, direct+substitution effects;
   dlnp_bev1 = dlnp_bev10 + dlnp_bev11; 
   dlnp_bev2 = dlnp_bev20 + dlnp_bev21; 

   *new full group price;
   p_bev1 = p_bev*(1+dlnp_bev1); 
   p_bev2 = p_bev*(1+dlnp_bev2); 

   *new price ignoring second order effect;
   p_bev10 = p_bev*(1+dlnp_bev10);
   p_bev20 = p_bev*(1+dlnp_bev20);
 
   q1 = q0#(J(ncol(bet),1,0)<>(J(ncol(bet),1,1)+H_u*(tax1/p0_oz/p))); 
   q2 = q0#(J(ncol(bet),1,0)<>(J(ncol(bet),1,1)+H_u*(tax2/p0_oz/p))); 

   *new per capita calories;
   engy1 = (calorie/8)`*q1;  
   engy2 = (calorie/8)`*q2;  

   *per capita tax burden;
   burd1 = q1`*tax1;           
   burd2 = q2`*tax2;           

   *per capita total ounces;
   tq0 = J(1,ncol(bet),1)*q0;  
   tq1 = J(1,ncol(bet),1)*q1;  
   tq2 = J(1,ncol(bet),1)*q2;      

   *per capita compensating variation;
   ub0 = (log(y) - (a0 + a1[n,]*lnp_bev + a2[n,]*lnp_oth + 0.5*(g11[n,]*lnp_bev*lnp_bev + g22[n,]*lnp_oth*lnp_oth) 
      + g12[n,]*lnp_bev*lnp_oth))/((exp(lnp_bev)**c1[n,])*(exp(lnp_oth)**c2[n,])); *Eq 4 of Deaton and Muellbauer;

   *partial cv including direct effect only;
   lny_10 = a0 + a1[n,]*log(p_bev10) + a2[n,]*lnp_oth + 0.5*(g11[n,]*log(p_bev10)*log(p_bev10) + g22[n,]*lnp_oth*lnp_oth) 
      + g12[n,]*log(p_bev10)*lnp_oth + ub0*(p_bev10**c1[n,])*(exp(lnp_oth)**c2[n,]);

   lny_20 = a0 + a1[n,]*log(p_bev20) + a2[n,]*lnp_oth + 0.5*(g11[n,]*log(p_bev20)*log(p_bev20) + g22[n,]*lnp_oth*lnp_oth) 
      + g12[n,]*log(p_bev20)*lnp_oth + ub0*(p_bev20**c1[n,])*(exp(lnp_oth)**c2[n,]);

   cv10 = (exp(lny_10) - y)/cpi;

   cv20 = (exp(lny_20) - y)/cpi;

   *full cv including direct and indirect price effects;
   lny_1 = a0 + a1[n,]*log(p_bev1) + a2[n,]*lnp_oth + 0.5*(g11[n,]*log(p_bev1)*log(p_bev1) + g22[n,]*lnp_oth*lnp_oth) 
      + g12[n,]*log(p_bev1)*lnp_oth + ub0*(p_bev1**c1[n,])*(exp(lnp_oth)**c2[n,]);

   lny_2 = a0 + a1[n,]*log(p_bev2) + a2[n,]*lnp_oth + 0.5*(g11[n,]*log(p_bev2)*log(p_bev2) + g22[n,]*lnp_oth*lnp_oth) 
      + g12[n,]*log(p_bev2)*lnp_oth + ub0*(p_bev2**c1[n,])*(exp(lnp_oth)**c2[n,]);

   cv1 = (exp(lny_1) - y)/cpi;

   cv2 = (exp(lny_2) - y)/cpi;

   *compensating variation per 1000 kcal reduced;
   cv1kcal = 1000*cv1/(engy0-engy1);
   cv2kcal = 1000*cv2/(engy0-engy2);

   *difference between cv1kcal and cv2kcal;
   dif_cvkcal = cv1kcal - cv2kcal; 

   *dead weight loss per 1000 kcal reduced;
   dwl1kcal = 1000*(cv1-burd1)/(engy0-engy1);
   dwl2kcal = 1000*(cv2-burd2)/(engy0-engy2);

   *difference between dwl1kcal and dwl2kcal;
   dif_dwlkcal = dwl1kcal - dwl2kcal; 

   if n = 1 then do;
    MAT = CL||H_u||E||w||p;   

    LB = 'MID'||'PID'||LB||'E'||'w'||'p';  

    create onemkte from MAT [colname=LB];
    append from MAT;
   end;

   sout[n,] = e_b||tq0||tq1||tq2||dlnp_bev10||dlnp_bev20||dlnp_bev11||dlnp_bev21||dlnp_bev1||dlnp_bev2||engy0||engy1||engy2||burd1||burd2||
              cv10||cv20||cv1||cv2||cv1kcal||cv2kcal||dif_cvkcal||dwl1kcal||dwl2kcal||dif_dwlkcal;
  
   hout[n,] = homo;
   aout[n,] = addup;

 end;

 homo_t = abs(mean(hout[2:&nsm+1,])/sqrt(var(hout[2:&nsm+1,])));
 addup_t = abs(mean(aout[2:&nsm+1,])/sqrt(var(aout[2:&nsm+1,])));

 MAT2 = (CL[1,1]||{1}||sout[1,]||mean(abs(hout[1,]`))||abs(aout[1,]))//
        (CL[1,1]||{2}||mean(sout[2:&nsm+1,])/sqrt(var(sout[2:&nsm+1,]))||mean(homo_t`)||addup_t);

 LB2 = 'MID'||'TYPE'||'e_b'||'tq0'||'tq1'||'tq2'||'dlnp10'||'dlnp20'||'dlnp11'||'dlnp21'||'dlnp1'||'dlnp2'||'engy0'||'engy1'||'engy2'||
       'burd1'||'burd2'||'cv10'||'cv20'||'cv1'||'cv2'||'cv1kcal'||'cv2kcal'||'dif_cvkcal'||'dwl1kcal'||'dwl2kcal'||'dif_dwlkcal'||
       'homo'||'addup';
  
 create onemktq from MAT2 [colname=LB2];
 append from MAT2;

 quit;

 data allmkte;
 set allmkte onemkte;
 run;

 data allmktq;
 set allmktq onemktq;
 run;

%end;

%mend;
