*plot own and cross price elasticities;
%macro plote;

data owne; set _null_; run;

data crosse; set _null_; run;

%do m=1 %to &NP;
data tempoe(rename=(PID&m=e));
set allmkte;
if PID ne &m then delete;
keep PID PID&m;
run;

data owne; set owne tempoe; run;

data tempce;
set allmkte;
if PID ne &m then delete;
drop PID&m E w p;
run;

proc transpose data = tempce out=tempce1;
by MID PID;
run;

data crosse; set crosse tempce1; run;
%end;

*calculate 1% and 99% tiles of own price elasticities;
proc univariate data = owne;
var e;
ods output quantiles = temp1;
run;

*create two macros variables to save the numeric values of 1% and 99%tile;
data _null_;
set temp1;
if quantile="99%" then call symput("_99pct",estimate);
if quantile="1%" then call symput("_1pct",estimate);
run;
%put &_1pct, &_99pct;

*restrict own elasticity plot to 98% of data;
data owne98;
set owne;
if &_1pct < e < &_99pct then output;
run;

ods graphics on / imagename="Own_e" imagefmt=pdf height=3.5in;
proc sgplot data = owne98;
histogram e;
xaxis label="Unconditional own-price elasticities";
run;

*identify observations with positive own price elasticities;
data owne;
set owne;
if e gt 0 then dum=1; else dum=0;
run;

data crosse; 
set crosse;
if COL1 = . then delete;
run;

*calculate 1% and 99% tiles of cross price elasticities;
proc univariate data = crosse;
var COL1;
ods output quantiles = temp2;
run;

*create two macros variables to save the numeric values of 1% and 99%tile;
data _null_;
set temp2;
if quantile="99%" then call symput("_99pct",estimate);
if quantile="1%" then call symput("_1pct",estimate);
run;
%put &_1pct, &_99pct;

*restrict own elasticity plot to 98% of data;
data crosse98;
set crosse;
if &_1pct < COL1 < &_99pct then output;
run;

ods graphics on / imagename="Cross_e" imagefmt=pdf height=3.5in;
proc sgplot data = crosse98;
histogram COL1;
xaxis label="Unconditional cross-price elasticities";
run;

*identify observations with negative cross price elasticities;
data crosse;
set crosse;
if COL1 lt 0 then dum=1; else dum=0;
run;

proc means data = owne mean;
title 'percent of positive own-price elasticities';
var dum;
run;

proc means data = crosse mean;
title 'percent of negative cross-price elasticities';
var dum;
run;

data allmktq;
set allmktq;
adlnp11 = abs(dlnp11);
adlnp21 = abs(dlnp21);
run;

proc means data = allmktq;
class type;
run;

%mend;
