*AJAE revision;
*Normalized by sum of budget shares;
*CREATE THE DISTANCE VARIABLES THAT ARE USED IN ESTIMATION;

libname sasdata "/vol1/project/0213096.NHLBI_price/ny/ajae/compile/normalize";

%include '/vol1/project/0213096.NHLBI_price/ny/ajae/macro/normalize/aspace.sas';
%include '/vol1/project/0213096.NHLBI_price/ny/ajae/macro/normalize/dalist.sas';
%include '/vol1/project/0213096.NHLBI_price/ny/ajae/macro/normalize/dalist2.sas';
%include '/vol1/project/0213096.NHLBI_price/ny/ajae/macro/normalize/calist.sas';
%include '/vol1/project/0213096.NHLBI_price/ny/ajae/macro/normalize/calist2.sas';
%include '/vol1/project/0213096.NHLBI_price/ny/ajae/macro/normalize/pr.sas';
%include '/vol1/project/0213096.NHLBI_price/ny/ajae/macro/normalize/nevopr.sas';

%let nda=%sysfunc(countw(%dalist));   %put &nda;  *# of equal discrete attributes;
%let nda2=%sysfunc(countw(%dalist2)); %put &nda2; *# of counter-equal discrete attributes;
%let nca=%sysfunc(countw(%calist));   %put &nca;  *# of one-way continuous attributes;
%let nca2=%sysfunc(countw(%calist2)); %put &nca2; *# of m-way continuous attributes;
%let cutw=0.000;

proc sort data = "/vol1/project/0213096.NHLBI_price/ny/ajae/compile/nevo_brand.sas7bdat" out = allbev;
by mktID QTR BRAND CAT PRODUCT;
run;

data allbev;
set allbev;
by mktID QTR;
retain MID;
 if first.QTR then MID+1;    *each market-quarter-week pair is considered as a market;
run;

proc summary data = allbev;
var DOL;
by MID;
output out = xallbev sum = X;
run;

data allbev;
merge allbev xallbev;
by MID;
w = DOL/X;
drop _TYPE_ _FREQ_;
proc sort;
by BRAND CAT PRODUCT;
run;

proc summary data = allbev;
var w;
by BRAND CAT PRODUCT;
output out = catch_small mean = wbar;
run;

data allbev;
merge allbev catch_small;
by BRAND CAT PRODUCT;
if w < &cutw then delete;  *delete censored sales and tiny market share products, if stkdum=0 the item is assumed to be stocked out;
icat7 = cat7;              *if there is a counter-equal attribute variable, here is where to define it;
icat10 = cat10;
nbottle = usize/bsize;
drop w X wbar _TYPE_ _FREQ_;
proc sort;
by mktID BRAND CAT PRODUCT;
run;

proc summary data = allbev;
var dol;
by mktID BRAND CAT PRODUCT;
output out = n_product n=number;
run;

*each product has to be in a city for at least two quarter-weeks due to use of fixed effects models;
data allbev;
merge allbev n_product;
by mktID BRAND CAT PRODUCT;
if number < 2 then delete;
drop number _TYPE_ _FREQ_;
proc sort;
by MID;
run;

proc summary data = allbev;
var dol;
by MID;
output out = xdata sum = x;
run;

data allbev;
merge allbev xdata;
by MID;
w = dol/x;
pcx = x/pop;      *per capita group expenditure;
lnx = log(pcx);
if QTR in (1,14,27,40,53) then qw=1;
else if QTR in (2,15,28,41,54) then qw=2;
else if QTR in (3,16,29,42,55) then qw=3;
else if QTR in (4,17,30,43,56) then qw=4;
else if QTR in (5,18,31,44,57) then qw=5;
else if QTR in (6,19,32,45,58) then qw=6;
else if QTR in (7,20,33,46,59) then qw=7;
else if QTR in (8,21,34,47,60) then qw=8;
else if QTR in (9,22,35,48,61) then qw=9;
else if QTR in (10,23,36,49,62) then qw=10;
else if QTR in (11,24,37,50,63) then qw=11;
else if QTR in (12,25,38,51,64) then qw=12;
else if QTR in (13,26,39,52,65) then qw=13;
drop _TYPE_ _FREQ_;
run;

proc summary data = allbev;
var pcx;
by MID mktID qw;
output out = midpcx mean = ;
run;

proc means data = midpcx noprint;
var pcx;
class mktID qw;
types mktID*qw;
output out = mktqwpcx sum = totpcx;
run;

proc sort data = allbev; by mktID qw; run;

data allbev;
merge allbev mktqwpcx;
by mktID qw;
ivlnx = log((totpcx-pcx)/(_freq_-1));  *average group expenditures in same QW in other years, same market as instrument;
drop _TYPE_ _FREQ_;
proc sort;
by BRAND CAT PRODUCT;
run;

*calculate mean budget shares and prices by brand;
proc summary data = allbev;
var dol vol;
output out = totalny sum = nydol nyvol;
run;

proc summary data = allbev;
var dol vol;
by BRAND CAT PRODUCT;
output out = byproduct sum = ;
run;

data totalny; set totalny; uno = 1; drop _TYPE_ _FREQ_; run;

data byproduct; set byproduct; uno = 1; drop _TYPE_ _FREQ_; run;

data avg_wp;
merge totalny byproduct;
by uno;
w0 = dol/nydol;
p0_oz = dol/vol;
w0p0_oz = w0*log(p0_oz);
keep BRAND CAT PRODUCT w0 p0_oz w0p0_oz;
run;

data avg_wp;
set avg_wp;
PID+1;
run;

proc summary data = avg_wp;
var w0p0_oz;
output out = avg_all sum = lnp0_oz_all;
run;

*get the average of base shares;
proc summary data = avg_wp;
var w0;
output out = avg_w0 mean = w00;
run;

data avg_all;
set avg_all;
drop _TYPE_ _FREQ_;
p0_oz_all = exp(lnp0_oz_all);  *average price of all beverages using stone price formula; 
uno = 1;
run;

data avg_w0;
set avg_w0;
drop _TYPE_ _FREQ_;
uno = 1;
run;

data allbev;
merge allbev avg_wp;
by BRAND CAT PRODUCT;
p = p_oz/p0_oz;
nevop = nevop_oz/p0_oz;
uno = 1;
drop w0p0_oz;
proc sort;
by mktID PID; 
run;

data allbev;
set allbev;
by mktID PID;
retain MPID;
if first.PID then MPID+1;
run;

data allbev;
merge allbev avg_all avg_w0;
by uno;
proc sort;
by PID MID;
run;

%aspace;

proc sort data = spacedata; by BRAND CAT PRODUCT; run;

data spacedata;
set spacedata;
wp = w*log(p);
w0p = w0*log(p);
w0nevop = w0*log(nevop);
proc sort;
by MID;
run;

*calculate stone prices;
proc summary data = spacedata;
var wp w0p w0nevop;
by MID;
output out = groupp sum = stonep laspep ivstonep;
run;

data spacedata;
merge spacedata groupp;
by MID;
drop _TYPE_ _FREQ_;
run;

proc reg data = spacedata;
model p = nevop;
run;

proc reg data = spacedata;
model stonep = ivstonep;
run;

proc reg data = spacedata;
model stonep = laspep;
run;

proc reg data = spacedata;
model laspep = ivstonep;
run;

proc reg data = spacedata;
model lnx = ivlnx;
run;

proc means data = spacedata mean N;
var p_oz w;
class PID;
run;

data sasdata.spacedata;
set spacedata;
drop wp w0p w0nevop stkdum nevostkdum
cat1-cat6 cat8 cat9 brand2 brand4-brand6 brand8 brand10 brand12 flavor flavor1-flavor34 scat1-scat16 
firm1-firm11 caff adcaff1 regular regular1 
diet diet1 calorie1 calorie2;
run;

proc export data=sasdata.spacedata outfile="/vol1/project/0213096.NHLBI_price/ny/ajae/compile/normalize/spacedata.dta" replace;
run;
