*Top Level Full AIDS Demand;

libname home "/vol1/project/0213096.NHLBI_price/ny/ajae/demand/top";
libname sasdata "/vol1/project/0213096.NHLBI_price/ny/ajae/compile";

%include '/vol1/project/0213096.NHLBI_price/ny/ajae/macro/thecs_aid.sas';

*according to 2010 CEX, 2.5 is the average number of persons per household;  
proc sql;
  create table top1 as 
  select mktid, qtr, qw, (sum(vol)/avg(pop)) as vol, (sum(dol)/avg(pop)) as dol, (avg(hhinc)/13/2.5) as y, avg(stonep) as lnp1,
         avg(laspep) as ivlnp1, (log(avg(cpi))-(avg(pcx)/(avg(hhinc)/13/2.5))*avg(stonep))/(1-avg(pcx)/(avg(hhinc)/13/2.5)) as lnp2,
         avg(pcx)/(avg(hhinc)/13/2.5) as w1, avg(cpi) as cpi
  from sasdata.spacedata
  group by mktid, qtr, qw;
  quit;

proc sql;
  create table top2 as
  select *, (log(cpi)-(avg(dol)/y)*ivlnp1)/(1-avg(dol)/y) as ivlnp2
  from top1;
  quit;

data top2;
set top2;
if mktid = 2  then mkt2 = 1;  else mkt2 = 0;
if mktid = 7  then mkt7 = 1;  else mkt7 = 0;
if mktid = 32 then mkt32 = 1; else mkt32 = 0; 
if mktid = 49 then mkt49 = 1; else mkt49 = 0;
if qtr <14 then yr07 = 1; else yr07 = 0;
if qtr > 13 and qtr < 27 then yr08 = 1; else yr08 = 0;
if qtr > 26 and qtr < 40 then yr09 = 1; else yr09 = 0;
if qtr > 39 and qtr < 53 then yr10 = 1; else yr10 = 0;
if qtr > 52 and qtr < 66 then yr11 = 1; else yr11 = 0;
if qw = 1  then qw1  = 1; else qw1  = 0;
if qw = 2  then qw2  = 1; else qw2  = 0;
if qw = 3  then qw3  = 1; else qw3  = 0;
if qw = 4  then qw4  = 1; else qw4  = 0;
if qw = 5  then qw5  = 1; else qw5  = 0;
if qw = 6  then qw6  = 1; else qw6  = 0;
if qw = 7  then qw7  = 1; else qw7  = 0;
if qw = 8  then qw8  = 1; else qw8  = 0;
if qw = 9  then qw9  = 1; else qw9  = 0;
if qw = 10 then qw10 = 1; else qw10 = 0;
if qw = 11 then qw11 = 1; else qw11 = 0;
if qw = 12 then qw12 = 1; else qw12 = 0;
if qw = 13 then qw13 = 1; else qw13 = 0;
run;

*estimate a full AIDS model for the top stage demand;
proc model data = top2;

endogenous w1 lnp1 lnp2;
exogenous y mkt2 mkt7 mkt32 mkt49 qw1-qw13 yr07-yr11 ivlnp1 ivlnp2;

parms
a10             0.020728
a1_mkt7          0.00025
a1_mkt32        -0.00067
a1_mkt49        -0.00022
a1_qw2           -0.0001
a1_qw3          -0.00004
a1_qw4          -0.00008
a1_qw5          0.000037
a1_qw6          0.000361
a1_qw7          0.000567
a1_qw8          0.000303
a1_qw9          0.000324
a1_qw10         -0.00011
a1_qw11         -0.00016
a1_qw12         -5.75E-6
a1_qw13         0.000053
a1_yr08         -0.00026
a1_yr09         -0.00034
a1_yr10         -0.00035
a1_yr11         -0.00043
g11             0.000081
c1              -0.00222
ap10            -0.00142
ap11            1.046107
ap20            6.035E-6
ap21            0.999814
;

%thecs_aid;

eq.g1 = a1 + g11*lnp1 + g12*lnp2 + c1*(log(y)-logp) - w1;

eq.g2 = ap10 + ap11*ivlnp1 - lnp1;

eq.g3 = ap20 + ap21*ivlnp2 - lnp2;

fit g1 g2 g3/fiml vardef = n converge=1.0e-6 maxit=30000 outest=parm_cov_h1 outcov;   *_h1 stands for alternative hypothesis;
run;

data parm_h1; set parm_cov_h1; if _N_ ne 1 then delete; 
drop _NAME_ _TYPE_ _STATUS_ _NUSED_ ap10 ap11 ap20 ap21;
run;

data cov_h1; set parm_cov_h1; if _N_ = 1 then delete; 
if _NAME_ in ("ap10","ap11","ap20","ap21") then delete;
drop _NAME_ _TYPE_ _STATUS_ _NUSED_ ap10 ap11 ap20 ap21;
run;

data top2_h1;
if _N_ = 1 then set parm_h1;
set top2;
%thecs_aid;
w1_hat = a1 + g11*lnp1 + g12*lnp2 + c1*(log(y)-logp);
e11 = -1 + (g11 - c1*(a1 + g11*lnp1 + g12*lnp2))/w1_hat;
e12 = (g12 - c1*(a2 + g12*lnp1 + g22*lnp2))/w1_hat;
e21 = (g12 - c2*(a1 + g11*lnp1 + g12*lnp2))/(1 - w1_hat);
e22 = -1 + (g22 - c2*(a2 + g12*lnp1 + g22*lnp2))/(1 - w1_hat);
e1 = 1 + c1/w1_hat;
e2 = 1 + c2/(1 - w1_hat);
h11 = e11 + w1_hat*e1;
h12 = e12 + (1 - w1_hat)*e1;
h21 = e21 + w1_hat*e2;
h22 = e22 + (1 - w1_hat)*e2;
ub0 = (log(y) - (a0 + a1*lnp1 + a2*lnp2 + 0.5*(g11*lnp1*lnp1 + g22*lnp2*lnp2) 
      + g12*lnp1*lnp2))/((exp(lnp1)**c1)*(exp(lnp2)**c2)); *Eq 4 of Deaton and Muellbauer;
lnp1_new = log(1.0756)+lnp1;
lny_new = a0 + a1*lnp1_new + a2*lnp2 + 0.5*(g11*lnp1_new*lnp1_new + g22*lnp2*lnp2) 
      + g12*lnp1_new*lnp2 + ub0*(exp(lnp1_new)**c1)*(exp(lnp2)**c2);
cv = exp(lny_new) - y;
run;

proc means data = top2_h1;
var e11 e12 e21 e22 e1 e2 h11 h12 h21 h22 cv;
title "Results from accounting for endogeneity";
run;

*test exogeneity of lnp1 and lnp2 because lnp1 has current budget shares in it;
proc model data = top2;

endogenous w1;
exogenous lnp1 lnp2 y mkt2 mkt7 mkt32 mkt49 qw1-qw13 yr07-yr11;

parms
a10             0.020728
a1_mkt7          0.00025
a1_mkt32        -0.00067
a1_mkt49        -0.00022
a1_qw2           -0.0001
a1_qw3          -0.00004
a1_qw4          -0.00008
a1_qw5          0.000037
a1_qw6          0.000361
a1_qw7          0.000567
a1_qw8          0.000303
a1_qw9          0.000324
a1_qw10         -0.00011
a1_qw11         -0.00016
a1_qw12         -5.75E-6
a1_qw13         0.000053
a1_yr08         -0.00026
a1_yr09         -0.00034
a1_yr10         -0.00035
a1_yr11         -0.00043
g11             0.000081
c1              -0.00222
;

%thecs_aid;

eq.g1 = a1 + g11*lnp1 + g12*lnp2 + c1*(log(y)-logp) - w1;

fit g1/fiml vardef = n converge=1.0e-6 maxit=30000 outest=parm_cov_h0 outcov;   *_h0 stands for null hypothesis (of price exogeneity);
run;

data parm_h0; set parm_cov_h0; if _N_ ne 1 then delete; 
drop _NAME_ _TYPE_ _STATUS_ _NUSED_;
run;

data cov_h0; set parm_cov_h0; if _N_ = 1 then delete; 
drop _NAME_ _TYPE_ _STATUS_ _NUSED_;
run;

data top2_h0;
if _N_ = 1 then set parm_h0;
set top2;
%thecs_aid;
w1_hat = a1 + g11*lnp1 + g12*lnp2 + c1*(log(y)-logp);
e11 = -1 + (g11 - c1*(a1 + g11*lnp1 + g12*lnp2))/w1_hat;
e12 = (g12 - c1*(a2 + g12*lnp1 + g22*lnp2))/w1_hat;
e21 = (g12 - c2*(a1 + g11*lnp1 + g12*lnp2))/(1 - w1_hat);
e22 = -1 + (g22 - c2*(a2 + g12*lnp1 + g22*lnp2))/(1 - w1_hat);
e1 = 1 + c1/w1_hat;
e2 = 1 + c2/(1 - w1_hat);
h11 = e11 + w1_hat*e1;
h12 = e12 + (1 - w1_hat)*e1;
h21 = e21 + w1_hat*e2;
h22 = e22 + (1 - w1_hat)*e2;
run;

proc means data = top2_h0;
var e11 e12 e21 e22 e1 e2 h11 h12 h21 h22;
title "Results without accounting for endogeneity";
run;

*Hausman test;
proc iml;
  use parm_h0; read all into parm_h0;  
  use parm_h1; read all into parm_h1;

  use cov_h0; read all into cov_h0;
  use cov_h1; read all into cov_h1;

  H = (parm_h1 - parm_h0)*inv(cov_h1 - cov_h0)*(parm_h1` - parm_h0`);
  DF = ncol(parm_h1);
  print H; print DF;
quit;

data home.parm_top; set parm_h1; run;

data home.cov_top; set cov_h1; run;



